bl_info = {
"name": "Save/Copy Logic", 
"version": (1, 0, 0),
"blender": (2, 76, 0),
"location": "Logic Editor > Properties > Save Logic Bricks Tab" ,
"description": "Save custom game logic setups. Important saving message in panel.",
"category": "Game Engine"} 
  
   
import bpy
import os
from bpy.props import StringProperty
from bpy.types import Header, Menu, Panel

######################################################################################
#Logic Setup Panel  
class LogicCopySetup(bpy.types.Panel):
	"""Creates a Panel in the Logic Editor properties window""" 
	bl_label = "Copy Logic Bricks"
	bl_options = {'DEFAULT_CLOSED'} 
	bl_idname = "OBJECT_copy_logic_setup"
	bl_space_type = 'LOGIC_EDITOR'
	bl_region_type = 'UI'
	

	def draw(self, context):
		layout = self.layout
		
		act_obj = context.active_object

		row = layout.row()  
		row.label("Important Tip!", icon='ERROR')
		row = layout.row() 
		row.operator("copylogic.tipmessage", text="Saving Tip", icon='INFO')
		row = layout.row()
		row = layout.row()   
		box = layout.box()
		box.label("Save Logic:")
		row = layout.row() 
		box.operator("object.copy_logic_bricks", text="Save Game Logic", icon='FILE_TICK')
		row = layout.row() 
		box.prop(act_obj, "logic_brick_copy")   
		row = layout.row()
		box.prop(act_obj, "save_sign")   
		row = layout.row()
		box = layout.box()
		box.label("Add Logic:")
		row = layout.row()
		box.prop(act_obj, "add_to_state", text="Add to State")
		row = layout.row()
		box.prop(act_obj, "change_name", text="Logic Names")
		row = layout.row()
		box.menu("LOGIC_copy_to_state", text="Logic")
		row = layout.row()  
#End of Logic Setup Panel 
######################################################################################
		 
		
class Copy_Links_State(bpy.types.Menu):   
	bl_label = "Logic to State"   
	bl_idname = "LOGIC_copy_to_state"
	
	
	def draw(self, context):
		layout = self.layout							 
			  
		
def LOGIC_Copy_Links_Add(self, context):
	self.layout.menu(Copy_Links_Add.bl_idname)
#End of Header/Menu Tab Button in Logic Editor
######################################################################################

class TipCopyOperator(bpy.types.Operator):
	bl_idname = "copylogic.tipmessage"
	bl_label = "Saving Tip"


	def invoke(self, context, event):
		wm = context.window_manager
		return wm.invoke_popup(self, width=675, height=750)
		return {'FINISHED'}

	def draw(self, context):

		layout = self.layout
		col = layout.column()
		col.label(text = "To save the links between the logic bricks. The user needs to tell the controller which sensors and actuators are connected.")
		col.label(text = "IF YOU DON'T DO THIS. The logic bricks will not be linked when you add them.")
		col.label(text = "To do this, you need to add each controller name that's connected, to the end of each sensor and actuator name, and end with a +.")
		col.label()
		col.label(text = "Examples:")
		col.label(text = "         If you have two rows of sensors and actuators connected to one controller. You need to name the bricks like this.")
		col.label(text = "         Sensor1+Hat+______________________Hat______________________Actuator1+Hat+")
		col.label(text = "                                                                         ^")
		col.label(text = "         Sensor2+Hat+_______________________^_______________________Actuator2+Hat+")
		col.label()
		col.label()
		col.label(text = "         If you have a sensor and/or actuator connected to multiple controllers. You need to name the bricks like this.")
		col.label(text = "         Sensor+Hat+Ball+Dog+______________________Hat______________________Actuator+Hat+")
		col.label(text = "         _______________________________________________Ball")
		col.label(text = "         _______________________________________________Dog")
		col.label()
		col.label(text = "Note:")
		col.label(text = "         When naming controllers I'd would make it a character or two, maybe use numbers, first as 1, second as 2, etc.")
		col.label(text = "         If you need to add the logic setup more than once. You need to remove the controller names from the sensors and actuators.")
		col.label(text = "         Also, you need to rename the controllers.")
		col.label(text = "         If you don't rename the bricks, it will not link or it will link to the previous controllers, sensors, or actuators.")
######################################################################################
			
class LogicCopyOperator(bpy.types.Operator):
	bl_idname = "copylogic.message"
	bl_label = "Saved"


	def invoke(self, context, event):
		wm = context.window_manager
		return wm.invoke_popup(self, width=500, height=50)
		return {'FINISHED'}

	def draw(self, context):
		logic_name = context.object.logic_brick_copy		
		logic_name = logic_name.replace(".", "")
		logic_name = logic_name.replace(" ", "_")

		layout = self.layout
		box = layout.box()
		box.label(text = "Game logic setup " + "'" + logic_name + "'" + " saved.", icon = "LOGIC")
		box.label(text = "Note:   Enable in User Preferences under Add Logic State tab!")

######################################################################################

class ErrorCopyOperator(bpy.types.Operator):
	bl_idname = "copylogic.errormessage"
	bl_label = "Error"


	def invoke(self, context, event):
		wm = context.window_manager
		return wm.invoke_popup(self, width=200, height=50)
		return {'FINISHED'}

	def draw(self, context):
		layout = self.layout
		layout.label(text = "This file already exists!", icon = "ERROR")
				
######################################################################################
#Start Save logic Brick Setup  
class SaveCopy(bpy.types.Operator):
	"""Save logic bricks"""  
	bl_idname = "object.copy_logic_bricks"
	bl_label = "Save Logic Bricks"

	
	def execute(self, context):
		
		
		def getfilepath():
			
		
			filepath = ""
			for p in bpy.utils.script_paths():
				filepath = os.path.join(p, 'addons')
				if os.path.exists(filepath):
					break
			return filepath

		logic_name = context.object.logic_brick_copy		
		logic_name = logic_name.replace(".", "")
		logic_name = logic_name.replace(" ", "_")

		try:
		# If it exists then report an error
			file = open(os.path.join(getfilepath(), logic_name + '.py'), 'r')
			file.close()
			bpy.ops.copylogic.errormessage('INVOKE_DEFAULT')
			return {'CANCELLED'}
		except IOError:
			# If it doesn't exists then write file  
			  if logic_name:   
				  logic_name = context.object.logic_brick_copy
				  logic_name = logic_name.replace(".", "")
				  logic_name = logic_name.replace(" ", "_")   
				  file = open(os.path.join(getfilepath(), logic_name + '.py'), 'w')
				  bpy.ops.copylogic.message('INVOKE_DEFAULT')



		#Start writing data to file.
		file.write("bl_info = {\n")
		file.write("\"name\": \"" + logic_name + "\", \n")
		file.write("\"version\": (1, 0, 0),\n")
		file.write("\"blender\": (2, 76, 0),\n")
		file.write("\"location\": \"Logic Editor > Properties > Copy Logic Bricks > Logic > " + logic_name + "\",\n")
		file.write("\"description\": \"Create " + logic_name + " game logic.\",\n")
		file.write("\"category\": \"Add Logic State\"}\n")
		file.write("\n")
		file.write("\n")
		file.write("import bpy")
		file.write("\n")
		file.write("\n")
		file.write("\n")
		file.write("class " + logic_name + "(bpy.types.Operator):\n")
		file.write("	bl_idname = \"object." + str.lower(logic_name) + "\"\n")
		file.write("	bl_label = \"" + logic_name + "\"\n")
		file.write("	bl_description = \"Add to active object " + logic_name + " game logic\"\n")
		file.write("\n")
		file.write("\n")
		file.write("	def execute(self, context):")
		#########################################################################

		file.write("\n")
		file.write("		act_obj = bpy.context.active_object")
		file.write("\n")
		file.write("		change_name = context.object.change_name")
		file.write("\n")
		file.write("		state_to_go_to = context.object.add_to_state")
		file.write("\n")
		file.write("\n")
		file.write("\n")	
		file.write("		#Logic Sensors ")
		file.write("\n")

		save_sign = context.object.save_sign
		if save_sign == '':
			save_sign = '+'

		act_obj = bpy.context.active_object
		active_state = act_obj.game.states_visible

		for i in act_obj.game.sensors:
			sens_name = i.name
			if save_sign in sens_name:   
			
			#Get Sensor and Type
				sensor = act_obj.game.sensors[-1]
				change_name = context.object.change_name
				state_to_go_to = context.object.add_to_state
				expanded = i.show_expanded  
				type = i.type


				#Get Armature Sensor Data If Sensor Type is = Armature
				if type == 'ARMATURE':
						
					name = i.name
					pin = i.pin
					active = i.active
					true_pulse = i.use_pulse_true_level
					false_pulse = i.use_pulse_false_level
					tick_skip = i.tick_skip
					use_level = i.use_level
					use_tap = i.use_tap
					invert = i.invert
					test_type = i.test_type
					#End Getting Armature Sensor Data
					

					#Start Writing Armature Data to File
					file.write("		bpy.ops.logic.sensor_add(type = " + "'" + type + "'", " name = " + "'" + name + "' " + "+ " + "\"" + "%s" + "\"" + " % " + "+ " + "change_name " + ")")
					file.write("\n")
					file.write("		sensor = act_obj.game.sensors"  + "["  '-1'  "]")
					file.write("\n")
					file.write("		sensor.show_expanded = "  + "%s" % expanded)
					file.write("\n")	
					file.write("		sensor.pin = "  + "%s" % pin)
					file.write("\n")
					file.write("		sensor.active = "  + "%s" % active)
					file.write("\n")
					file.write("		sensor.use_pulse_true_level = "  + "%s" % true_pulse)
					file.write("\n")
					file.write("		sensor.use_pulse_false_level = "  + "%s" % false_pulse)
					file.write("\n")
					file.write("		sensor.tick_skip = " + "%s" % tick_skip)
					file.write("\n")
					file.write("		sensor.use_level = " + "%s" % use_level)
					file.write("\n")
					file.write("		sensor.use_tap = " + "%s" % use_tap)
					file.write("\n")		
					file.write("		sensor.invert = " + "%s" % invert)
					file.write("\n")		
					file.write("		sensor.test_type = " + "'"  + "%s" % test_type + "'")
					file.write("\n")		

					if test_type != 'STATECHG':
						value = i.value
						
						file.write("		sensor.value = " + "%s" % value)
						file.write("\n")				 
						file.write("\n")
				#End Writing Armature Sensor Data
				#########################################################################


				#########################################################################
				#Get Actuator Sensor Data If Sensor Type is = Actuator
				if type == 'ACTUATOR':
						
					name = i.name
					pin = i.pin
					active = i.active
					true_pulse = i.use_pulse_true_level
					false_pulse = i.use_pulse_false_level
					tick_skip = i.tick_skip
					use_level = i.use_level
					use_tap = i.use_tap
					invert = i.invert
					actuator = i.actuator
					#End Getting Actuator Sensor Data
					

					#Start Writing Actuator Data to File
					file.write("		bpy.ops.logic.sensor_add(type = " + "'" + type + "'," " name = " + "'" + name + "' " + "+ " + "\"" + "%s" + "\"" + " % " + "+ " + "change_name " + ")")
					file.write("\n")
					file.write("		sensor = act_obj.game.sensors"  + "["  '-1'  "]")
					file.write("\n")
					file.write("		sensor.show_expanded = "  + "%s" % expanded)
					file.write("\n")	
					file.write("		sensor.pin = "  + "%s" % pin)
					file.write("\n")
					file.write("		sensor.active = "  + "%s" % active)
					file.write("\n")
					file.write("		sensor.use_pulse_true_level = "  + "%s" % true_pulse)
					file.write("\n")
					file.write("		sensor.use_pulse_false_level = "  + "%s" % false_pulse)
					file.write("\n")
					file.write("		sensor.tick_skip = " + "%s" % tick_skip)
					file.write("\n")
					file.write("		sensor.use_level = " + "%s" % use_level)
					file.write("\n")
					file.write("		sensor.use_tap = " + "%s" % use_tap)
					file.write("\n")
					file.write("		sensor.invert = " + "%s" % invert)
					file.write("\n")
					file.write("		sensor.actuator = " + "'"  + "%s" % actuator + "'")
					file.write("\n")	
					file.write("\n")
				#End Writing Actuator Sensor Data
				#########################################################################


				#########################################################################
				#Get Always Sensor Data If Sensor Type is = Always
				if type == 'ALWAYS':
						
					name = i.name
					pin = i.pin
					active = i.active
					true_pulse = i.use_pulse_true_level
					false_pulse = i.use_pulse_false_level
					tick_skip = i.tick_skip
					use_level = i.use_level
					use_tap = i.use_tap
					invert = i.invert
					#End Getting Always Sensor Data


					#Start Writing Always Data to File
					file.write("		bpy.ops.logic.sensor_add(type = " + "'" + type + "'," " name = " + "'" + name + "' " + "+ " + "\"" + "%s" + "\"" + " % " + "+ " + "change_name " + ")")
					file.write("\n")
					file.write("		sensor = act_obj.game.sensors"  + "["  '-1'  "]")
					file.write("\n")
					file.write("		sensor.show_expanded = "  + "%s" % expanded)
					file.write("\n")	
					file.write("		sensor.pin = "  + "%s" % pin)
					file.write("\n")
					file.write("		sensor.active = "  + "%s" % active)
					file.write("\n")
					file.write("		sensor.use_pulse_true_level = "  + "%s" % true_pulse)
					file.write("\n")
					file.write("		sensor.use_pulse_false_level = "  + "%s" % false_pulse)
					file.write("\n")
					file.write("		sensor.tick_skip = " + "%s" % tick_skip)
					file.write("\n")
					file.write("		sensor.use_level = " + "%s" % use_level)
					file.write("\n")
					file.write("		sensor.use_tap = " + "%s" % use_tap)
					file.write("\n")
					file.write("		sensor.invert = " + "%s" % invert)
					file.write("\n")
					file.write("\n")
				#End Writing Always Sensor Data
				#########################################################################
				
				
				#########################################################################
				#Get Collision Sensor Data If Sensor Type is = Collision
				if type == 'COLLISION':
						
					name = i.name
					pin = i.pin
					active = i.active
					true_pulse = i.use_pulse_true_level
					false_pulse = i.use_pulse_false_level
					tick_skip = i.tick_skip
					use_level = i.use_level
					use_tap = i.use_tap
					invert = i.invert
					use_pulse = i.use_pulse
					use_material = i.use_material   
					#End Getting Collision Sensor Data  

				
					#Start Writing Collision Data to File
					file.write("		bpy.ops.logic.sensor_add(type = " + "'" + type + "'," " name = " + "'" + name + "' " + "+ " + "\"" + "%s" + "\"" + " % " + "+ " + "change_name " + ")")
					file.write("\n")
					file.write("		sensor = act_obj.game.sensors"  + "["  '-1'  "]")
					file.write("\n")
					file.write("		sensor.show_expanded = "  + "%s" % expanded)
					file.write("\n")	
					file.write("		sensor.pin = "  + "%s" % pin)
					file.write("\n")
					file.write("		sensor.active = "  + "%s" % active)
					file.write("\n")
					file.write("		sensor.use_pulse_true_level = "  + "%s" % true_pulse)
					file.write("\n")
					file.write("		sensor.use_pulse_false_level = "  + "%s" % false_pulse)
					file.write("\n")
					file.write("		sensor.tick_skip = " + "%s" % tick_skip)
					file.write("\n")
					file.write("		sensor.use_level = " + "%s" % use_level)
					file.write("\n")
					file.write("		sensor.use_tap = " + "%s" % use_tap)
					file.write("\n")
					file.write("		sensor.invert = " + "%s" % invert)
					file.write("\n")
					file.write("		sensor.use_pulse = " + "%s" % use_pulse)
					file.write("\n")
					file.write("		sensor.use_material = " + "%s" % use_material)
					file.write("\n")		   
					
					
					if use_material == True:
						material = i.material
						
						file.write("		sensor.material = " + "'"  + "%s" % material + "'")
						file.write("\n")				 
						file.write("\n")
								
					if use_material == False:
						property = i.property
						
						file.write("		sensor.property = " + "'"  + "%s" % property + "'")
						file.write("\n")
						file.write("\n")		 
				#End Writing Collision Sensor Data
				#########################################################################   
				

				#########################################################################
				#Get Delay Sensor Data If Sensor Type is = Delay
				if type == 'DELAY':
						
					name = i.name
					pin = i.pin
					active = i.active
					true_pulse = i.use_pulse_true_level
					false_pulse = i.use_pulse_false_level
					tick_skip = i.tick_skip
					use_level = i.use_level
					use_tap = i.use_tap
					invert = i.invert
					delay = i.delay
					duration = i.duration
					use_repeat = i.use_repeat   
					#End Getting Delay Sensor Data 


					#Start Writing Delay Data to File
					file.write("		bpy.ops.logic.sensor_add(type = " + "'" + type + "'," " name = " + "'" + name + "' " + "+ " + "\"" + "%s" + "\"" + " % " + "+ " + "change_name " + ")")
					file.write("\n")
					file.write("		sensor = act_obj.game.sensors"  + "["  '-1'  "]")
					file.write("\n")
					file.write("		sensor.show_expanded = "  + "%s" % expanded)
					file.write("\n")	
					file.write("		sensor.pin = "  + "%s" % pin)
					file.write("\n")
					file.write("		sensor.active = "  + "%s" % active)
					file.write("\n")
					file.write("		sensor.use_pulse_true_level = "  + "%s" % true_pulse)
					file.write("\n")
					file.write("		sensor.use_pulse_false_level = "  + "%s" % false_pulse)
					file.write("\n")
					file.write("		sensor.tick_skip = " + "%s" % tick_skip)
					file.write("\n")
					file.write("		sensor.use_level = " + "%s" % use_level)
					file.write("\n")
					file.write("		sensor.use_tap = " + "%s" % use_tap)
					file.write("\n")
					file.write("		sensor.invert = " + "%s" % invert)
					file.write("\n")
					file.write("		sensor.delay = " + "%s" % delay)
					file.write("\n")
					file.write("		sensor.duration = " + "%s" % duration)
					file.write("\n")
					file.write("		sensor.use_repeat = " + "%s" % use_repeat)
					file.write("\n")			
					file.write("\n")
				#End Writing Delay Sensor Data
				######################################################################### 


				#########################################################################
				#Get Joystick Sensor Data If Sensor Type is = Joystick
				if type == 'JOYSTICK':
											
					name = i.name
					pin = i.pin
					active = i.active
					true_pulse = i.use_pulse_true_level
					false_pulse = i.use_pulse_false_level
					tick_skip = i.tick_skip
					use_level = i.use_level
					use_tap = i.use_tap
					invert = i.invert
					joystick_index = i.joystick_index
					event_type = i.event_type
					use_all_events = i.use_all_events   
					#End Getting Joystick Sensor Data
					   
				
					#Start Writing Joystick Data to File
					file.write("		bpy.ops.logic.sensor_add(type = " + "'" + type + "'," " name = " + "'" + name + "' " + "+ " + "\"" + "%s" + "\"" + " % " + "+ " + "change_name " + ")")
					file.write("\n")
					file.write("		sensor = act_obj.game.sensors"  + "["  '-1'  "]")
					file.write("\n")
					file.write("		sensor.show_expanded = "  + "%s" % expanded)
					file.write("\n")	
					file.write("		sensor.pin = "  + "%s" % pin)
					file.write("\n")
					file.write("		sensor.active = "  + "%s" % active)
					file.write("\n")
					file.write("		sensor.use_pulse_true_level = "  + "%s" % true_pulse)
					file.write("\n")
					file.write("		sensor.use_pulse_false_level = "  + "%s" % false_pulse)
					file.write("\n")
					file.write("		sensor.tick_skip = " + "%s" % tick_skip)
					file.write("\n")
					file.write("		sensor.use_level = " + "%s" % use_level)
					file.write("\n")
					file.write("		sensor.use_tap = " + "%s" % use_tap)
					file.write("\n")
					file.write("		sensor.invert = " + "%s" % invert)
					file.write("\n")
					file.write("		sensor.joystick_index = "  + "%s" % joystick_index)
					file.write("\n")
					file.write("		sensor.event_type = " + "'"  + "%s" % event_type + "'")
					file.write("\n")
					file.write("		sensor.use_all_events = " + "%s" % use_all_events)
					file.write("\n")								
					
					if event_type == 'BUTTON':
						button_number = i.button_number
						
						file.write("		sensor.button_number = " + "%s" % button_number)
						file.write("\n")		
						file.write("\n") 
								
					if event_type == 'AXIS':
						axis_number = i.axis_number
						axis_threshold = i.axis_threshold
						axis_direction = i.axis_direction 
						
						file.write("		sensor.axis_number = " + "%s" % axis_number)
						file.write("\n")		
						file.write("		sensor.axis_threshold = " + "%s" % axis_threshold)
						file.write("\n")				   
						file.write("		sensor.axis_direction = " + "'"  + "%s" % axis_direction + "'")
						file.write("\n")
						file.write("\n") 
													 
					if event_type == 'HAT':
						hat_number = i.hat_number
						hat_direction = i.hat_direction
								
						file.write("		sensor.hat_number = " + "%s" % hat_number)
						file.write("\n")		
						file.write("		sensor.hat_direction = " + "'"  + "%s" % hat_direction + "'")
						file.write("\n")				   
						file.write("\n")
							 
					if event_type == 'AXIS_SINGLE':
						single_axis_number = i.single_axis_number
						axis_threshold = i.axis_threshold
						
						file.write("		sensor.single_axis_number = " + "%s" % single_axis_number)
						file.write("\n")		
						file.write("		sensor.axis_threshold = "  + "%s" % axis_threshold)
						file.write("\n")				 
						file.write("\n")				 
				#End Writing Joystick Sensor Data
				######################################################################### 
				
				
				#########################################################################
				#Get Keyboard Sensor Data If Sensor Type is = Keyboard
				if type == 'KEYBOARD':
						
					name = i.name
					pin = i.pin
					active = i.active
					true_pulse = i.use_pulse_true_level
					false_pulse = i.use_pulse_false_level
					tick_skip = i.tick_skip
					use_level = i.use_level
					use_tap = i.use_tap
					invert = i.invert
					key = i.key
					use_all_keys = i.use_all_keys
					modifier_key_1 = i.modifier_key_1
					modifier_key_2 = i.modifier_key_2   
					log = i.log
					target = i.target   
					#End Getting Keyboard Sensor Data 
					
				
					#Start Writing Keyboard Data to File
					file.write("		bpy.ops.logic.sensor_add(type = " + "'" + type + "'," " name = " + "'" + name + "' " + "+ " + "\"" + "%s" + "\"" + " % " + "+ " + "change_name " + ")")
					file.write("\n")
					file.write("		sensor = act_obj.game.sensors"  + "["  '-1'  "]")
					file.write("\n")
					file.write("		sensor.show_expanded = "  + "%s" % expanded)
					file.write("\n")	
					file.write("		sensor.pin = "  + "%s" % pin)
					file.write("\n")
					file.write("		sensor.active = "  + "%s" % active)
					file.write("\n")
					file.write("		sensor.use_pulse_true_level = "  + "%s" % true_pulse)
					file.write("\n")
					file.write("		sensor.use_pulse_false_level = "  + "%s" % false_pulse)
					file.write("\n")
					file.write("		sensor.tick_skip = " + "%s" % tick_skip)
					file.write("\n")
					file.write("		sensor.use_level = " + "%s" % use_level)
					file.write("\n")
					file.write("		sensor.use_tap = " + "%s" % use_tap)
					file.write("\n")
					file.write("		sensor.invert = " + "%s" % invert)
					file.write("\n")
					file.write("		sensor.key = " + "'"  + "%s" % key + "'")
					file.write("\n")
					file.write("		sensor.use_all_keys = " + "%s" % use_all_keys)
					file.write("\n")
					file.write("		sensor.modifier_key_1 = " + "'"  + "%s" % modifier_key_1 + "'")
					file.write("\n")
					file.write("		sensor.modifier_key_2 = " + "'"  + "%s" % modifier_key_2 + "'")
					file.write("\n")
					file.write("		sensor.log = " + "'"  + "%s" % log + "'")
					file.write("\n")
					file.write("		sensor.target = " + "'"  + "%s" % target + "'")
					file.write("\n")						   
					file.write("\n")
				#End Writing Keyboard Sensor Data
				#########################################################################   
				

				#########################################################################
				#Get Message Sensor Data If Sensor Type is = Message
				if type == 'MESSAGE':
						
					name = i.name
					pin = i.pin
					active = i.active
					true_pulse = i.use_pulse_true_level
					false_pulse = i.use_pulse_false_level
					tick_skip = i.tick_skip
					use_level = i.use_level
					use_tap = i.use_tap
					invert = i.invert
					subject = i.subject
					#End Getting Message Sensor Data
				
				
					#Start Writing Message Data to File
					file.write("		bpy.ops.logic.sensor_add(type = " + "'" + type + "'," " name = " + "'" + name + "' " + "+ " + "\"" + "%s" + "\"" + " % " + "+ " + "change_name " + ")")
					file.write("\n")
					file.write("		sensor = act_obj.game.sensors"  + "["  '-1'  "]")
					file.write("\n")
					file.write("		sensor.show_expanded = "  + "%s" % expanded)
					file.write("\n")	
					file.write("		sensor.pin = "  + "%s" % pin)
					file.write("\n")
					file.write("		sensor.active = "  + "%s" % active)
					file.write("\n")
					file.write("		sensor.use_pulse_true_level = "  + "%s" % true_pulse)
					file.write("\n")
					file.write("		sensor.use_pulse_false_level = "  + "%s" % false_pulse)
					file.write("\n")
					file.write("		sensor.tick_skip = " + "%s" % tick_skip)
					file.write("\n")
					file.write("		sensor.use_level = " + "%s" % use_level)
					file.write("\n")
					file.write("		sensor.use_tap = " + "%s" % use_tap)
					file.write("\n")
					file.write("		sensor.invert = " + "%s" % invert)
					file.write("\n")
					file.write("		sensor.subject = " + "'"  + "%s" % subject + "'")
					file.write("\n")
					file.write("\n")		 
				#End Writing Message Sensor Data
				#########################################################################   


				#########################################################################
				#Get Mouse Sensor Data If Sensor Type is = Mouse
				if type == 'MOUSE':
						
					name = i.name
					pin = i.pin
					active = i.active
					true_pulse = i.use_pulse_true_level
					false_pulse = i.use_pulse_false_level
					tick_skip = i.tick_skip
					use_level = i.use_level
					use_tap = i.use_tap
					invert = i.invert
					mouse_event = i.mouse_event   
					#End Getting Mouse Sensor Data 


					#Start Writing Mouse Data to File
					file.write("		bpy.ops.logic.sensor_add(type = " + "'" + type + "'," " name = " + "'" + name + "' " + "+ " + "\"" + "%s" + "\"" + " % " + "+ " + "change_name " + ")")
					file.write("\n")
					file.write("		sensor = act_obj.game.sensors"  + "["  '-1'  "]")
					file.write("\n")
					file.write("		sensor.show_expanded = "  + "%s" % expanded)
					file.write("\n")	
					file.write("		sensor.pin = "  + "%s" % pin)
					file.write("\n")
					file.write("		sensor.active = "  + "%s" % active)
					file.write("\n")
					file.write("		sensor.use_pulse_true_level = "  + "%s" % true_pulse)
					file.write("\n")
					file.write("		sensor.use_pulse_false_level = "  + "%s" % false_pulse)
					file.write("\n")
					file.write("		sensor.tick_skip = " + "%s" % tick_skip)
					file.write("\n")
					file.write("		sensor.use_level = " + "%s" % use_level)
					file.write("\n")
					file.write("		sensor.use_tap = " + "%s" % use_tap)
					file.write("\n")
					file.write("		sensor.invert = " + "%s" % invert)
					file.write("\n")
					file.write("		sensor.mouse_event = " + "'"  + "%s" % mouse_event + "'")
					file.write("\n")						   
				
					if mouse_event == 'MOUSEOVERANY':
						use_pulse = sensor.use_pulse
						use_material = sensor.use_material  
						use_x_ray = sensor.use_x_ray
								
						file.write("		sensor.use_pulse = " + "%s" % use_pulse)
						file.write("\n")
						file.write("		sensor.use_material = " + "'"  + "%s" % use_material + "'")
						file.write("\n")							
						file.write("		sensor.use_x_ray = "  + "%s" % use_x_ray)
						file.write("\n")
								
						if use_material == 'MATERIAL':
							material = sensor.material
							
							file.write("		sensor.material = " + "'"  + "%s" % material + "'")
							file.write("\n")				 
							
						if use_material == 'PROPERTY':
							property = sensor.property
							
							file.write("		sensor.property = " + "'"  + "%s" % property + "'")
							file.write("\n")
							file.write("\n")										  
				#End Writing Mouse Sensor Data
				######################################################################### 


				#########################################################################
				#Get Near Sensor Data If Sensor Type is = Near
				if type == 'NEAR':
						
					name = i.name
					pin = i.pin
					active = i.active
					true_pulse = i.use_pulse_true_level
					false_pulse = i.use_pulse_false_level
					tick_skip = i.tick_skip
					use_level = i.use_level
					use_tap = i.use_tap
					invert = i.invert
					property = i.property
					distance = i.distance
					reset_distance = i.reset_distance   
					#End Getting Near Sensor Data 


					#Start Writing Near Data to File
					file.write("		bpy.ops.logic.sensor_add(type = " + "'" + type + "'," " name = " + "'" + name + "' " + "+ " + "\"" + "%s" + "\"" + " % " + "+ " + "change_name " + ")")
					file.write("\n")
					file.write("		sensor = act_obj.game.sensors"  + "["  '-1'  "]")
					file.write("\n")
					file.write("		sensor.show_expanded = "  + "%s" % expanded)
					file.write("\n")	
					file.write("		sensor.pin = "  + "%s" % pin)
					file.write("\n")
					file.write("		sensor.active = "  + "%s" % active)
					file.write("\n")
					file.write("		sensor.use_pulse_true_level = "  + "%s" % true_pulse)
					file.write("\n")
					file.write("		sensor.use_pulse_false_level = "  + "%s" % false_pulse)
					file.write("\n")
					file.write("		sensor.tick_skip = " + "%s" % tick_skip)
					file.write("\n")
					file.write("		sensor.use_level = " + "%s" % use_level)
					file.write("\n")
					file.write("		sensor.use_tap = " + "%s" % use_tap)
					file.write("\n")
					file.write("		sensor.invert = " + "%s" % invert)
					file.write("\n")
					file.write("		sensor.property = " + "'"  + "%s" % property + "'")
					file.write("\n")
					file.write("		sensor.distance = " + "%s" % distance)
					file.write("\n")
					file.write("		sensor.reset_distance = " + "%s" % reset_distance)
					file.write("\n")							   
					file.write("\n")
				#End Writing Near Sensor Data
				######################################################################### 


				#########################################################################
				#Get Property Data If Sensor Type is = Property
				if type == 'PROPERTY':
						
					name = i.name
					pin = i.pin
					active = i.active
					true_pulse = i.use_pulse_true_level
					false_pulse = i.use_pulse_false_level
					tick_skip = i.tick_skip
					use_level = i.use_level
					use_tap = i.use_tap
					invert = i.invert
					evaluation_type = i.evaluation_type
					property = i.property
					value = i.value  
					#End Getting Property Sensor Data 


					#Start Writing Property Data to File
					file.write("		bpy.ops.logic.sensor_add(type = " + "'" + type + "'," " name = " + "'" + name + "' " + "+ " + "\"" + "%s" + "\"" + " % " + "+ " + "change_name " + ")")
					file.write("\n")
					file.write("		sensor = act_obj.game.sensors"  + "["  '-1'  "]")
					file.write("\n")
					file.write("		sensor.show_expanded = "  + "%s" % expanded)
					file.write("\n")	
					file.write("		sensor.pin = "  + "%s" % pin)
					file.write("\n")
					file.write("		sensor.active = "  + "%s" % active)
					file.write("\n")
					file.write("		sensor.use_pulse_true_level = "  + "%s" % true_pulse)
					file.write("\n")
					file.write("		sensor.use_pulse_false_level = "  + "%s" % false_pulse)
					file.write("\n")
					file.write("		sensor.tick_skip = " + "%s" % tick_skip)
					file.write("\n")
					file.write("		sensor.use_level = " + "%s" % use_level)
					file.write("\n")
					file.write("		sensor.use_tap = " + "%s" % use_tap)
					file.write("\n")
					file.write("		sensor.invert = " + "%s" % invert)
					file.write("\n")
					file.write("		sensor.evaluation_type = " + "'"  + "%s" % evaluation_type + "'")
					file.write("\n")
					file.write("		sensor.property = " + "'"  + "%s" % property + "'")
					file.write("\n")
					file.write("		sensor.value = " + "'"  + "%s" % value + "'")
					file.write("\n")							   
					file.write("\n")
				#End Writing Property Sensor Data
				######################################################################### 


				#########################################################################
				#Get Radar Data If Sensor Type is = Radar
				if type == 'RADAR':
						
					name = i.name
					pin = i.pin
					active = i.active
					true_pulse = i.use_pulse_true_level
					false_pulse = i.use_pulse_false_level
					tick_skip = i.tick_skip
					use_level = i.use_level
					use_tap = i.use_tap
					invert = i.invert
					property = i.property
					axis = i.axis   
					angle = i.angle  
					distance = i.distance   
					#End Getting Radar Sensor Data 

				
					#Start Writing Radar Data to File
					file.write("		bpy.ops.logic.sensor_add(type = " + "'" + type + "'," " name = " + "'" + name + "' " + "+ " + "\"" + "%s" + "\"" + " % " + "+ " + "change_name " + ")")
					file.write("\n")
					file.write("		sensor = act_obj.game.sensors"  + "["  '-1'  "]")
					file.write("\n")
					file.write("		sensor.show_expanded = "  + "%s" % expanded)
					file.write("\n")	
					file.write("		sensor.pin = "  + "%s" % pin)
					file.write("\n")
					file.write("		sensor.active = "  + "%s" % active)
					file.write("\n")
					file.write("		sensor.use_pulse_true_level = "  + "%s" % true_pulse)
					file.write("\n")
					file.write("		sensor.use_pulse_false_level = "  + "%s" % false_pulse)
					file.write("\n")
					file.write("		sensor.tick_skip = " + "%s" % tick_skip)
					file.write("\n")
					file.write("		sensor.use_level = " + "%s" % use_level)
					file.write("\n")
					file.write("		sensor.use_tap = " + "%s" % use_tap)
					file.write("\n")
					file.write("		sensor.invert = " + "%s" % invert)
					file.write("\n")
					file.write("		sensor.property = " + "'"  + "%s" % property + "'")
					file.write("\n")
					file.write("		sensor.axis = " + "'"  + "%s" % axis + "'")
					file.write("\n")		
					file.write("		sensor.angle = "  + "%s" % angle)
					file.write("\n")
					file.write("		sensor.distance = "  + "%s" % distance)
					file.write("\n")								   
					file.write("\n")
				#End Writing Radar Sensor Data
				######################################################################### 


				#########################################################################
				#Get Random Data If Sensor Type is = Random
				if type == 'RANDOM':
						
					name = i.name
					pin = i.pin
					active = i.active
					true_pulse = i.use_pulse_true_level
					false_pulse = i.use_pulse_false_level
					tick_skip = i.tick_skip
					use_level = i.use_level
					use_tap = i.use_tap
					invert = i.invert
					seed = i.seed   
					#End Getting Random Sensor Data 


					#Start Writing Random Data to File
					file.write("		bpy.ops.logic.sensor_add(type = " + "'" + type + "'," " name = " + "'" + name + "' " + "+ " + "\"" + "%s" + "\"" + " % " + "+ " + "change_name " + ")")
					file.write("\n")
					file.write("		sensor = act_obj.game.sensors"  + "["  '-1'  "]")
					file.write("\n")
					file.write("		sensor.show_expanded = "  + "%s" % expanded)
					file.write("\n")	
					file.write("		sensor.pin = "  + "%s" % pin)
					file.write("\n")
					file.write("		sensor.active = "  + "%s" % active)
					file.write("\n")
					file.write("		sensor.use_pulse_true_level = "  + "%s" % true_pulse)
					file.write("\n")
					file.write("		sensor.use_pulse_false_level = "  + "%s" % false_pulse)
					file.write("\n")
					file.write("		sensor.tick_skip = " + "%s" % tick_skip)
					file.write("\n")
					file.write("		sensor.use_level = " + "%s" % use_level)
					file.write("\n")
					file.write("		sensor.use_tap = " + "%s" % use_tap)
					file.write("\n")
					file.write("		sensor.invert = " + "%s" % invert)
					file.write("\n")
					file.write("		sensor.seed = "  + "%s" % seed)
					file.write("\n")								   
					file.write("\n")
				#End Writing Random Sensor Data
				######################################################################### 


				#########################################################################
				#Get Ray Data If Sensor Type is = Ray
				if type == 'RAY':
						
					name = i.name
					pin = i.pin
					active = i.active
					true_pulse = i.use_pulse_true_level
					false_pulse = i.use_pulse_false_level
					tick_skip = i.tick_skip
					use_level = i.use_level
					use_tap = i.use_tap
					invert = i.invert
					ray_type = i.ray_type
					axis = i.axis
					range = i.range 
					use_x_ray = i.use_x_ray  
					#End Getting Ray Sensor Data 
				
				
					#Start Writing Ray Data to File
					file.write("		bpy.ops.logic.sensor_add(type = " + "'" + type + "'," " name = " + "'" + name + "' " + "+ " + "\"" + "%s" + "\"" + " % " + "+ " + "change_name " + ")")
					file.write("\n")
					file.write("		sensor = act_obj.game.sensors"  + "["  '-1'  "]")
					file.write("\n")
					file.write("		sensor.show_expanded = "  + "%s" % expanded)
					file.write("\n")	
					file.write("		sensor.pin = "  + "%s" % pin)
					file.write("\n")
					file.write("		sensor.active = "  + "%s" % active)
					file.write("\n")
					file.write("		sensor.use_pulse_true_level = "  + "%s" % true_pulse)
					file.write("\n")
					file.write("		sensor.use_pulse_false_level = "  + "%s" % false_pulse)
					file.write("\n")
					file.write("		sensor.tick_skip = " + "%s" % tick_skip)
					file.write("\n")
					file.write("		sensor.use_level = " + "%s" % use_level)
					file.write("\n")
					file.write("		sensor.use_tap = " + "%s" % use_tap)
					file.write("\n")
					file.write("		sensor.invert = " + "%s" % invert)
					file.write("\n")
					file.write("		sensor.ray_type = " + "'"  + "%s" % ray_type + "'")
					file.write("\n")
					file.write("		sensor.axis = " + "'"  + "%s" % axis + "'")
					file.write("\n")	
					file.write("		sensor.range = "  + "%s" % range)
					file.write("\n")	
					file.write("		sensor.use_x_ray = "  + "%s" % use_x_ray)
					file.write("\n")											

					if ray_type == 'MATERIAL':
						material = i.material
						
						file.write("		sensor.material = " + "'"  + "%s" % material + "'")
						file.write("\n")				 
						file.write("\n")
								
					if ray_type == 'PROPERTY':
						property = i.property
						
						file.write("		sensor.property = " + "'"  + "%s" % property + "'")
						file.write("\n")
						file.write("\n") 
		#End Writing Ray Sensor Data
		#########################################################################
		file.write("\n")
		file.write("		#Logic Controllers")
		file.write("\n")
		#Game Logic Controllers
		#########################################################################
		act_obj = bpy.context.active_object
		for k in act_obj.game.controllers:
			name = k.name
			if save_sign in name:
	
				controller = act_obj.game.controllers[-1]   
				change_name = context.object.change_name				
				#Get Controller Data
				expanded = k.show_expanded  
				type = k.type   
				use_priority = k.use_priority
				active = k.active
				#End Getting Controller Data
				
			   
				#Start Writing Controller Data to File

				file.write("		state_to_go_to = context.object.add_to_state")
				file.write("\n")
				file.write("\n")  
				file.write("		bpy.ops.logic.controller_add(type = " + "'" + type + "'," " name = " + "'" + name + "' " + "+ " + "\"" + "%s" + "\"" + " % " + "+ " + "change_name " + ")")
				file.write("\n")	
				file.write("		controller = act_obj.game.controllers"  + "["  '-1'  "]")
				file.write("\n")
				file.write("		controller.show_expanded = "  + "%s" % expanded)
				file.write("\n")		   
				file.write("		controller.states = state_to_go_to")
				file.write("\n")	 
				file.write("		controller.use_priority = "  + "%s" % use_priority)
				file.write("\n")
				file.write("		controller.active = "  + "%s" % active)
				file.write("\n")	
				file.write("\n")	
				
				if type == 'EXPRESSION':
					expression = k.expression   
				   
					file.write("		controller.expression = " + "'"  + "%s" % expression + "'")
					file.write("\n")
					file.write("\n")						

				if type == 'PYTHON':
					mode = k.mode
					
					file.write("		controller.mode = " + "'"  + "%s" % mode + "'")
					file.write("\n")
					file.write("\n")		   
						
					if mode == 'MODULE':
						module = k.module
						use_debug = k.use_debug
						
						file.write("		controller.module = " + "'"  + "%s" % module + "'")
						file.write("\n")					
						file.write("		controller.use_debug = "  + "%s" % use_debug)
						file.write("\n")
						file.write("\n")															
		#End Writing Controller Data
		#########################################################################
		file.write("\n")	
		file.write("		#Logic Actuators")
		file.write("\n")
		#Game Logic Actuators
		#########################################################################
		act_obj = bpy.context.active_object
		for i in act_obj.game.actuators:   
			act_name = i.name
			if save_sign in act_name:
					
				#Get Actuator and Type
				actuator = act_obj.game.actuators[-1]
				change_name = context.object.change_name				
				expanded = i.show_expanded  
				type = i.type


				if type == 'ARMATURE':
						
					name = i.name
					pin = i.pin
					active = i.active
					mode = i.mode
					#End Getting Armature Actuator Data
					

					#Start Writing Armature Data to File
					file.write("		bpy.ops.logic.actuator_add(type = " + "'" + type + "'," " name = " + "'" + name + "' " + "+ " + "\"" + "%s" + "\"" + " % " + "+ " + "change_name " + ")")
					file.write("\n")
					file.write("		actuator = act_obj.game.actuators"  + "["  '-1'  "]")
					file.write("\n")
					file.write("		actuator.show_expanded = "  + "%s" % expanded)
					file.write("\n")	
					file.write("		actuator.pin = "  + "%s" % pin)
					file.write("\n")
					file.write("		actuator.active = "  + "%s" % active)
					file.write("\n")
					file.write("		actuator.mode = " + "'"  + "%s" % mode + "'")
					file.write("\n")
					file.write("\n")

					if mode == 'SETWEIGHT':
						weight = i.weight
						
						file.write("		actuator.weight = " + "%s" % weight)
						file.write("\n")				 
						file.write("\n")

					if mode == 'SETINFLUENCE':
						influence = i.influence
						
						file.write("		actuator.influence = " + "%s" % influence)
						file.write("\n")				 
						file.write("\n")
				#End Writing Armature Actuator Data
				#########################################################################


				#########################################################################
				#Get Action Actuator Data If Actuator Type is = Action
				if type == 'ACTION':
						
					name = i.name
					pin = i.pin
					active = i.active
					play_mode = i.play_mode
					use_force = i.use_force
					use_additive = i.use_additive
					use_local = i.use_local
					action = i.action
					use_continue = i.use_continue_last_frame
					frame_start = i.frame_start
					frame_end = i.frame_end   
					apply_children = i.apply_to_children		
					blend_in = i.frame_blend_in  
					priority = i.priority   
					layer = i.layer
					layer_weight = i.layer_weight
					blend_mode = i.blend_mode   
					frame_property = i.frame_property			
					#End Getting Action Actuator Data
					

					#Start Writing Action Data to File
					file.write("		bpy.ops.logic.actuator_add(type = " + "'" + type + "'," " name = " + "'" + name + "' " + "+ " + "\"" + "%s" + "\"" + " % " + "+ " + "change_name " + ")")
					file.write("\n")
					file.write("		actuator = act_obj.game.actuators"  + "["  '-1'  "]")
					file.write("\n")
					file.write("		actuator.show_expanded = "  + "%s" % expanded)
					file.write("\n")	
					file.write("		actuator.pin = "  + "%s" % pin)
					file.write("\n")
					file.write("		actuator.active = "  + "%s" % active)
					file.write("\n")
					file.write("		actuator.play_mode = " + "'"  + "%s" % play_mode + "'")
					file.write("\n")
					file.write("		actuator.use_force = "  + "%s" % use_force)
					file.write("\n")
					file.write("		actuator.use_additive = "  + "%s" % use_additive)
					file.write("\n")
					file.write("		actuator.use_local = " + "%s" % use_local)
					file.write("\n")
					file.write("		actuator.action = " + "%s" % action)
					file.write("\n")
					file.write("		actuator.use_continue_last_frame = " + "%s" % use_continue)
					file.write("\n")
					file.write("		actuator.frame_start = " + "%s" % frame_start)
					file.write("\n")
					file.write("		actuator.frame_end = " + "%s" % frame_end)
					file.write("\n")
					file.write("		actuator.apply_to_children = " + "%s" % apply_children)
					file.write("\n")
					file.write("		actuator.frame_blend_in = " + "%s" % blend_in)
					file.write("\n")
					file.write("		actuator.priority = " + "%s" % priority)
					file.write("\n")
					file.write("		actuator.layer = " + "%s" % layer)
					file.write("\n")
					file.write("		actuator.layer_weight = " + "%s" % layer_weight)
					file.write("\n")
					file.write("		actuator.blend_mode = " + "'"  + "%s" % blend_mode + "'")
					file.write("\n")
					file.write("		actuator.frame_property = " + "'"  + "%s" % frame_property + "'")   
					file.write("\n")
					file.write("\n")		
					
					if play_mode == 'PROPERTY':
						property = i.property
						
						file.write("		actuator.property = " + "'"  + "%s" % property + "'")
						file.write("\n")
						file.write("\n")			
				#End Writing Action Actuator Data
				#########################################################################


				#########################################################################
				#Get Camera Actuator Data If Actuator Type is = Camera
				if type == 'CAMERA':
						
					name = i.name
					pin = i.pin
					active = i.active
					height = i.height
					axis = i.axis
					min = i.min
					max = i.max
					damping = i.damping   
					#End Getting Camera Actuator Data
					

					#Start Writing Camera Data to File
					file.write("		bpy.ops.logic.actuator_add(type = " + "'" + type + "'," " name = " + "'" + name + "' " + "+ " + "\"" + "%s" + "\"" + " % " + "+ " + "change_name " + ")")
					file.write("\n")
					file.write("		actuator = act_obj.game.actuators"  + "["  '-1'  "]")
					file.write("\n")
					file.write("		actuator.show_expanded = "  + "%s" % expanded)
					file.write("\n")	
					file.write("		actuator.pin = "  + "%s" % pin)
					file.write("\n")
					file.write("		actuator.active = "  + "%s" % active)
					file.write("\n")
					file.write("		actuator.height = "  + "%s" % height)
					file.write("\n")
					file.write("		actuator.axis = " + "'"  + "%s" % axis + "'")
					file.write("\n")
					file.write("		actuator.min = " + "%s" % min)
					file.write("\n")
					file.write("		actuator.max = " + "%s" % max)
					file.write("\n")
					file.write("		actuator.damping = " + "%s" % damping)  
					file.write("\n")
					file.write("\n")					
				#End Writing Camera Actuator Data
				#########################################################################						  
					   
							
				#########################################################################
				#Get Constraint Actuator Data If Actuator Type is = Constraint
				if type == 'CONSTRAINT':
						
					name = i.name
					pin = i.pin
					active = i.active
					mode = i.mode		  
					#End Getting Constraint Actuator Data
					

					#Start Writing Constraint Data to File
					file.write("		bpy.ops.logic.actuator_add(type = " + "'" + type + "'," " name = " + "'" + name + "' " + "+ " + "\"" + "%s" + "\"" + " % " + "+ " + "change_name " + ")")
					file.write("\n")
					file.write("		actuator = act_obj.game.actuators"  + "["  '-1'  "]")
					file.write("\n")
					file.write("		actuator.show_expanded = "  + "%s" % expanded)
					file.write("\n")	
					file.write("		actuator.pin = "  + "%s" % pin)
					file.write("\n")
					file.write("		actuator.active = "  + "%s" % active)
					file.write("\n")
					file.write("		actuator.mode = " + "'"  + "%s" % mode + "'")   
					file.write("\n")
					
					if mode == 'LOC':
						limit = i.limit
						limit_min = i.limit_min
						limit_max = i.limit_max
						damping = i.damping
															
						file.write("		actuator.limit = " + "'"  + "%s" % limit + "'")
						file.write("\n")
						file.write("		actuator.limit_min = "  + "%s" % limit_min)
						file.write("\n")
						file.write("		actuator.limit_max = "  + "%s" % limit_max)
						file.write("\n")
						file.write("		actuator.damping = "  + "%s" % damping)   
						file.write("\n")			
						file.write("\n")
						
					if mode == 'DIST':
						direction = i.direction
						use_local = i.use_local
						use_normal = i.use_normal
						use_force = i.use_force_distance
						range = i.range   
						distance = i.distance
						use_material_detect = i.use_material_detect   
						damping = i.damping   
						use_persistent = i.use_persistent
						time = i.time   
						damping_rotation = i.damping_rotation   
																								
						file.write("		actuator.direction = " + "'"  + "%s" % direction + "'")
						file.write("\n")
						file.write("		actuator.use_local = "  + "%s" % use_local)
						file.write("\n")
						file.write("		actuator.use_normal = "  + "%s" % use_normal)
						file.write("\n")
						file.write("		actuator.use_force_distance = "  + "%s" % use_force)
						file.write("\n")
						file.write("		actuator.range = "  + "%s" % range)
						file.write("\n")
						file.write("		actuator.distance = "  + "%s" % distance)
						file.write("\n")
						file.write("		actuator.use_material_detect = "  + "%s" % use_material_detect)   
						file.write("\n")
						file.write("		actuator.damping = "  + "%s" % damping)
						file.write("\n")
						file.write("		actuator.use_persistent = "  + "%s" % use_persistent)
						file.write("\n")
						file.write("		actuator.time = "  + "%s" % time)
						file.write("\n")						
						file.write("		actuator.damping_rotation = "  + "%s" % damping_rotation)					 
						file.write("\n")			
											
						if use_material_detect == False:
							property = i.property
							
							file.write("		actuator.property = " + "'"  + "%s" % property + "'")
							file.write("\n")				
							file.write("\n")				
							
						if use_material_detect == True:
							material = i.material
							
							file.write("		actuator.material = " + "'"  + "%s" % material + "'")
							file.write("\n")				
							file.write("\n")
							
					if mode == 'ORI':
						axis_direction = i.direction_axis_pos
						damping = i.damping
						time = i.time   
						rot_max_x = i.rotation_max.x
						rot_max_y = i.rotation_max.y
						rot_max_z = i.rotation_max.z						
						angle_min = i.angle_min
						angle_max = i.angle_max   
															
						file.write("		actuator.direction_axis_pos = " + "'"  + "%s" % axis_direction + "'")
						file.write("\n")
						file.write("		actuator.damping = "  + "%s" % damping)
						file.write("\n")
						file.write("		actuator.time = "  + "%s" % time)
						file.write("\n")
						file.write("		actuator.rotation_max.x = "  + "%s" % rot_max_x)
						file.write("\n")
						file.write("		actuator.rotation_max.y = "  + "%s" % rot_max_y)
						file.write("\n")
						file.write("		actuator.rotation_max.z = "  + "%s" % rot_max_z)
						file.write("\n")									
						file.write("		actuator.angle_min = "  + "%s" % angle_min)
						file.write("\n")
						file.write("		actuator.angle_max = "  + "%s" % angle_max)  
						file.write("\n")			
						file.write("\n")
												
					if mode == 'FH':
						fh_damping = i.fh_damping
						fh_height = i.fh_height
						rot_fh = i.use_fh_paralel_axis  
						axis_direction = i.direction_axis
						fh_force = i.fh_force
						fh_normal = i.use_fh_normal
						use_material_detect = i.use_material_detect   
						use_persistent = i.use_persistent
						time = i.time
						damping_rotation = i.damping_rotation							 
															
						file.write("		actuator.fh_damping = "  + "%s" % fh_damping)
						file.write("\n")
						file.write("		actuator.fh_height = "  + "%s" % fh_height)
						file.write("\n")
						file.write("		actuator.use_fh_paralel_axis = "  + "%s" % rot_fh)
						file.write("\n")
						file.write("		actuator.direction_axis = " + "'"  + "%s" % axis_direction + "'")
						file.write("\n")
						file.write("		actuator.fh_force = "  + "%s" % fh_force)
						file.write("\n")									
						file.write("		actuator.use_fh_normal = "  + "%s" % fh_normal)
						file.write("\n")
						file.write("		actuator.use_material_detect = "  + "%s" % use_material_detect)
						file.write("\n")
						file.write("		actuator.use_persistent = "  + "%s" % use_persistent)
						file.write("\n")
						file.write("		actuator.time = "  + "%s" % time)
						file.write("\n")
						file.write("		actuator.damping_rotation = "  + "%s" % damping_rotation)				   
						file.write("\n")			
						
						if use_material_detect == False:
							property = i.property
							
							file.write("		actuator.property = " + "'"  + "%s" % property + "'")
							file.write("\n")				
							file.write("\n")				
							
						if use_material_detect == True:
							material = i.material
							
							file.write("		actuator.material = " + "'"  + "%s" % material + "'")
							file.write("\n")				
							file.write("\n")										
				#End Writing Constraint Actuator Data
				#########################################################################
				
				
				#########################################################################
				#Get Edit Object Actuator Data If Actuator Type is = Edit Object
				if type == 'EDIT_OBJECT':
						
					name = i.name
					pin = i.pin
					active = i.active
					mode = i.mode		  
					#End Getting Edit Object Actuator Data
					

					#Start Writing Edit Object Data to File
					file.write("		bpy.ops.logic.actuator_add(type = " + "'" + type + "'," " name = " + "'" + name + "' " + "+ " + "\"" + "%s" + "\"" + " % " + "+ " + "change_name " + ")")
					file.write("\n")
					file.write("		actuator = act_obj.game.actuators"  + "["  '-1'  "]")
					file.write("\n")
					file.write("		actuator.show_expanded = "  + "%s" % expanded)
					file.write("\n")	
					file.write("		actuator.pin = "  + "%s" % pin)
					file.write("\n")
					file.write("		actuator.active = "  + "%s" % active)
					file.write("\n")
					file.write("		actuator.mode = " + "'"  + "%s" % mode + "'")   
					file.write("\n")		

					if mode == 'ADDOBJECT':
						time = i.time
						linear_velocity_x = i.linear_velocity.x
						linear_velocity_y = i.linear_velocity.y
						linear_velocity_z = i.linear_velocity.z
						local_linear_velocity = i.use_local_linear_velocity   
						angular_velocity_x = i.angular_velocity.x   
						angular_velocity_y = i.angular_velocity.y
						angular_velocity_z = i.angular_velocity.z	 
						local_angular_velocity = i.use_local_angular_velocity
																		
						file.write("		actuator.time = "  + "%s" % time)
						file.write("\n")
						file.write("		actuator.linear_velocity.x = "  + "%s" % linear_velocity_x)
						file.write("\n")
						file.write("		actuator.linear_velocity.y = "  + "%s" % linear_velocity_y)
						file.write("\n")
						file.write("		actuator.linear_velocity.z = "  + "%s" % linear_velocity_z)
						file.write("\n")
						file.write("		actuator.use_local_linear_velocity = "  + "%s" % local_linear_velocity)
						file.write("\n")
						file.write("		actuator.angular_velocity.x = "  + "%s" % angular_velocity_x)
						file.write("\n")
						file.write("		actuator.angular_velocity.y = "  + "%s" % angular_velocity_y)
						file.write("\n")
						file.write("		actuator.angular_velocity.z = "  + "%s" % angular_velocity_z)
						file.write("\n")
						file.write("		actuator.use_local_angular_velocity = "  + "%s" % local_angular_velocity)   
						file.write("\n")			
						file.write("\n")

					if mode == 'REPLACEMESH':
						use_display = i.use_replace_display_mesh
						use_physics = i.use_replace_physics_mesh
																		
						file.write("		actuator.use_replace_display_mesh = "  + "%s" % use_display)
						file.write("\n")
						file.write("		actuator.use_replace_physics_mesh = "  + "%s" % use_physics)
						file.write("\n")
						file.write("\n")

					if mode == 'TRACKTO':
						time = i.time
						use_3d = i.use_3d_tracking
						up_axis = i.up_axis
						track_axis = i.track_axis	 
																		
						file.write("		actuator.time = "  + "%s" % time)
						file.write("\n")
						file.write("		actuator.use_3d_tracking = "  + "%s" % use_3d)
						file.write("\n")
						file.write("		actuator.up_axis = " + "'"  + "%s" % up_axis + "'")
						file.write("\n")
						file.write("		actuator.track_axis = " + "'"  + "%s" % track_axis + "'")	 
						file.write("\n")
						file.write("\n")

					if mode == 'DYNAMICS':
						dynamic_operation = i.dynamic_operation 
																		
						file.write("		actuator.dynamic_operation = " + "'"  + "%s" % dynamic_operation + "'")   
						file.write("\n")
						file.write("\n")
						
						if dynamic_operation == 'SETMASS':
							mass = i.mass   
																			
							file.write("		actuator.mass = "  + "%s" % mass)	 
							file.write("\n")
							file.write("\n")			
				#End Writing Edit Object Actuator Data
				#########################################################################
				

				#########################################################################
				#Get Filter 2D Actuator Data If Actuator Type is = Filter 2D
				if type == 'FILTER_2D':
						
					name = i.name
					pin = i.pin
					active = i.active
					mode = i.mode
					filter_pass = i.filter_pass  
					#End Getting Filter 2D Actuator Data
					

					#Start Writing Filter 2D Data to File
					file.write("		bpy.ops.logic.actuator_add(type = " + "'" + type + "'," " name = " + "'" + name + "' " + "+ " + "\"" + "%s" + "\"" + " % " + "+ " + "change_name " + ")")
					file.write("\n")
					file.write("		actuator = act_obj.game.actuators"  + "["  '-1'  "]")
					file.write("\n")
					file.write("		actuator.show_expanded = "  + "%s" % expanded)
					file.write("\n")	
					file.write("		actuator.pin = "  + "%s" % pin)
					file.write("\n")
					file.write("		actuator.active = "  + "%s" % active)
					file.write("\n")
					file.write("		actuator.mode = " + "'"  + "%s" % mode + "'")
					file.write("\n")
					file.write("		actuator.filter_pass = "  + "%s" % filter_pass) 
					file.write("\n")
					file.write("\n")
					
					if mode == 'MOTIONBLUR':
						motion_blur_factor = i.motion_blur_factor
						use_motion_blur = i.use_motion_blur 
																		
						file.write("		actuator.motion_blur_factor = "  + "%s" % motion_blur_factor)	 
						file.write("\n")
						file.write("		actuator.use_motion_blur = "  + "%s" % use_motion_blur)   
						file.write("\n")
						file.write("\n")
				#End Writing Filter 2D Actuator Data
				#########################################################################


				#########################################################################
				#Get Game Actuator Data If Actuator Type is = Game
				if type == 'GAME':
						
					name = i.name
					pin = i.pin
					active = i.active
					mode = i.mode		  
					#End Getting Game Actuator Data
					

					#Start Writing Game Data to File
					file.write("		bpy.ops.logic.actuator_add(type = " + "'" + type + "'," " name = " + "'" + name + "' " + "+ " + "\"" + "%s" + "\"" + " % " + "+ " + "change_name " + ")")
					file.write("\n")
					file.write("		actuator = act_obj.game.actuators"  + "["  '-1'  "]")
					file.write("\n")
					file.write("		actuator.show_expanded = "  + "%s" % expanded)
					file.write("\n")	
					file.write("		actuator.pin = "  + "%s" % pin)
					file.write("\n")
					file.write("		actuator.active = "  + "%s" % active)
					file.write("\n")
					file.write("		actuator.mode = " + "'"  + "%s" % mode + "'")   
					file.write("\n")
					file.write("\n")
				#End Writing Game Actuator Data
				#########################################################################   
					
					
				#########################################################################
				#Get Message Actuator Data If Actuator Type is = Message
				if type == 'MESSAGE':
						
					name = i.name
					pin = i.pin
					active = i.active
					subject = i.subject
					body_type = i.body_type  
					#End Getting Message Actuator Data
					

					#Start Writing Message Data to File
					file.write("		bpy.ops.logic.actuator_add(type = " + "'" + type + "'," " name = " + "'" + name + "' " + "+ " + "\"" + "%s" + "\"" + " % " + "+ " + "change_name " + ")")
					file.write("\n")
					file.write("		actuator = act_obj.game.actuators"  + "["  '-1'  "]")
					file.write("\n")
					file.write("		actuator.show_expanded = "  + "%s" % expanded)
					file.write("\n")	
					file.write("		actuator.pin = "  + "%s" % pin)
					file.write("\n")
					file.write("		actuator.active = "  + "%s" % active)
					file.write("\n")
					file.write("		actuator.subject = " + "'"  + "%s" % subject + "'") 
					file.write("\n")
					
					if body_type == 'TEXT':
						body_message = i.body_message			  
																		
						file.write("		actuator.body_message = " + "'"  + "%s" % body_message + "'")   
						file.write("\n")
						file.write("\n")
						
					if body_type == 'PROPERTY':
						body_property = i.body_property   
																		
						file.write("		actuator.body_property = " + "'"  + "%s" % body_property + "'")   
						file.write("\n")
						file.write("\n")								
				#End Writing Message Actuator Data
				#########################################################################
				
				
				#########################################################################
				#Get Mouse Actuator Data If Actuator Type is = Mouse
				if type == 'MOUSE':
						
					name = i.name
					pin = i.pin
					active = i.active
					mode = i.mode		  
					#End Getting Mouse Actuator Data
					

					#Start Writing Mouse Data to File
					file.write("		bpy.ops.logic.actuator_add(type = " + "'" + type + "'," " name = " + "'" + name + "' " + "+ " + "\"" + "%s" + "\"" + " % " + "+ " + "change_name " + ")")
					file.write("\n")
					file.write("		actuator = act_obj.game.actuators"  + "["  '-1'  "]")
					file.write("\n")
					file.write("		actuator.show_expanded = "  + "%s" % expanded)
					file.write("\n")	
					file.write("		actuator.pin = "  + "%s" % pin)
					file.write("\n")
					file.write("		actuator.active = "  + "%s" % active)
					file.write("\n")				
					file.write("		actuator.mode = " + "'"  + "%s" % mode + "'")   
					file.write("\n")
					
					if mode == 'VISIBILITY':
						visible = i.visible   
																		
						file.write("		actuator.visible = "  + "%s" % visible)   
						file.write("\n")
						file.write("\n")
						
					if mode == 'LOOK':
						use_axis_x = i.use_axis_x
						sensitivity_x = i.sensitivity_x   
						threshold_x = i.threshold_x   
						min_x = i.min_x   
						max_x = i.max_x   
						object_axis_x = i.object_axis_x
						local_x = i.local_x   
						reset_x = i.reset_x   
						use_axis_y = i.use_axis_y
						sensitivity_y = i.sensitivity_y   
						threshold_y = i.threshold_y   
						min_y = i.min_y   
						max_y = i.max_y   
						object_axis_y = i.object_axis_y
						local_y = i.local_y   
						reset_y = i.reset_y   
																		
						file.write("		actuator.use_axis_x = "  + "%s" % use_axis_x)
						file.write("\n")
						file.write("		actuator.sensitivity_x = "  + "%s" % sensitivity_x)   
						file.write("\n")
						file.write("		actuator.threshold_x = "  + "%s" % threshold_x)   
						file.write("\n")
						file.write("		actuator.min_x = "  + "%s" % min_x)   
						file.write("\n")
						file.write("		actuator.max_x = "  + "%s" % max_x)
						file.write("\n")
						file.write("		actuator.object_axis_x = " + "'"  + "%s" % object_axis_x + "'")   
						file.write("\n")
						file.write("		actuator.local_x = "  + "%s" % local_x)
						file.write("\n")
						file.write("		actuator.reset_x = "  + "%s" % reset_x)   
						file.write("\n")
						file.write("		actuator.use_axis_y = "  + "%s" % use_axis_y)
						file.write("\n")
						file.write("		actuator.sensitivity_y = "  + "%s" % sensitivity_y)   
						file.write("\n")
						file.write("		actuator.threshold_y = "  + "%s" % threshold_y)   
						file.write("\n")
						file.write("		actuator.min_y = "  + "%s" % min_y)   
						file.write("\n")
						file.write("		actuator.max_y = "  + "%s" % max_y)
						file.write("\n")
						file.write("		actuator.object_axis_y = " + "'"  + "%s" % object_axis_y + "'")   
						file.write("\n")
						file.write("		actuator.local_y = "  + "%s" % local_y)
						file.write("\n")
						file.write("		actuator.reset_y = "  + "%s" % reset_y)  
						file.write("\n")
						file.write("\n")		
				#End Writing Mouse Actuator Data
				#########################################################################   
					
					
				#########################################################################
				#Get Motion Actuator Data If Actuator Type is = Motion
				if type == 'MOTION':
						
					name = i.name
					pin = i.pin
					active = i.active
					mode = i.mode
					physics_type = act_obj.game.physics_type							 
					#End Getting Motion Actuator Data
					

					#Start Writing Motion Data to File
					file.write("		bpy.ops.logic.actuator_add(type = " + "'" + type + "'," " name = " + "'" + name + "' " + "+ " + "\"" + "%s" + "\"" + " % " + "+ " + "change_name " + ")")
					file.write("\n")
					file.write("		actuator = act_obj.game.actuators"  + "["  '-1'  "]")
					file.write("\n")
					file.write("		actuator.show_expanded = "  + "%s" % expanded)
					file.write("\n")	
					file.write("		actuator.pin = "  + "%s" % pin)
					file.write("\n")
					file.write("		actuator.active = "  + "%s" % active)
					file.write("\n")				
					file.write("		actuator.mode = " + "'"  + "%s" % mode + "'")   
					file.write("\n")		
						
					if mode == 'OBJECT_NORMAL':
						offset_location_x = i.offset_location.x
						offset_location_y = i.offset_location.y   
						offset_location_z = i.offset_location.z
						use_local_location = i.use_local_location   
						offset_rotation_x = i.offset_rotation.x
						offset_rotation_y = i.offset_rotation.y   
						offset_rotation_z = i.offset_rotation.z
						use_local_rotation = i.use_local_rotation   
																													 
						file.write("		actuator.offset_location.x = "  + "%s" % offset_location_x)   
						file.write("\n")
						file.write("		actuator.offset_location.y = "  + "%s" % offset_location_y)
						file.write("\n")
						file.write("		actuator.offset_location.z = "  + "%s" % offset_location_z)
						file.write("\n")
						file.write("		actuator.use_local_location = "  + "%s" % use_local_location)
						file.write("\n")
						file.write("		actuator.offset_rotation.x = "  + "%s" % offset_rotation_x)
						file.write("\n")
						file.write("		actuator.offset_rotation.y = "  + "%s" % offset_rotation_y)
						file.write("\n")
						file.write("		actuator.offset_rotation.z = "  + "%s" % offset_rotation_z)   
						file.write("\n")
						file.write("		actuator.use_local_rotation = "  + "%s" % use_local_rotation)   
						file.write("\n")
						file.write("\n")

						if physics_type == 'DYNAMIC':
							force_x = i.force.x
							force_y = i.force.y   
							force_z = i.force.z
							use_local_force = i.use_local_force   
							torque_x = i.torque.x
							torque_y = i.torque.y   
							torque_z = i.torque.z
							use_local_torque = i.use_local_torque
							linear_velocity_x = i.linear_velocity.x
							linear_velocity_y = i.linear_velocity.y   
							linear_velocity_z = i.linear_velocity.z
							use_local_linear_velocity = i.use_local_linear_velocity
							use_add_linear_velocity = i.use_add_linear_velocity
							angular_velocity_x = i.angular_velocity.x
							angular_velocity_y = i.angular_velocity.y   
							angular_velocity_z = i.angular_velocity.z
							use_local_angular_velocity = i.use_local_angular_velocity
							damping = i.damping 
														
							file.write("		actuator.force.x = "  + "%s" % force_x)   
							file.write("\n")
							file.write("		actuator.force.y = "  + "%s" % force_y)
							file.write("\n")
							file.write("		actuator.force.z = "  + "%s" % force_z)
							file.write("\n")
							file.write("		actuator.use_local_force = "  + "%s" % use_local_force)
							file.write("\n")
							file.write("		actuator.torque.x = "  + "%s" % torque_x)
							file.write("\n")
							file.write("		actuator.torque.y = "  + "%s" % torque_y)
							file.write("\n")
							file.write("		actuator.torque.z = "  + "%s" % torque_z)   
							file.write("\n")
							file.write("		actuator.use_local_torque = "  + "%s" % use_local_torque)
							file.write("\n")				
							file.write("		actuator.linear_velocity.x = "  + "%s" % linear_velocity_x)
							file.write("\n")
							file.write("		actuator.linear_velocity.y = "  + "%s" % linear_velocity_y)
							file.write("\n")
							file.write("		actuator.linear_velocity.z = "  + "%s" % linear_velocity_z)   
							file.write("\n")
							file.write("		actuator.use_local_linear_velocity = "  + "%s" % use_local_linear_velocity)
							file.write("\n")
							file.write("		actuator.use_add_linear_velocity = "  + "%s" % use_add_linear_velocity) 
							file.write("\n")
							file.write("		actuator.angular_velocity.x = "  + "%s" % angular_velocity_x)
							file.write("\n")
							file.write("		actuator.angular_velocity.y = "  + "%s" % angular_velocity_y)
							file.write("\n")
							file.write("		actuator.angular_velocity.z = "  + "%s" % angular_velocity_z)   
							file.write("\n")
							file.write("		actuator.use_local_angular_velocity = "  + "%s" % use_local_angular_velocity)
							file.write("\n")			
							file.write("		actuator.damping = "  + "%s" % damping)   
							file.write("\n")			
							file.write("\n")			
						
					if mode == 'OBJECT_SERVO':
						linear_velocity_x = i.linear_velocity.x
						linear_velocity_y = i.linear_velocity.y
						linear_velocity_z = i.linear_velocity.z
						use_local_linear_velocity = i.use_local_linear_velocity
						use_servo_limit_x = i.use_servo_limit_x   
						force_max_x = i.force_max_x
						force_min_x = i.force_min_x
						use_servo_limit_y = i.use_servo_limit_y   
						force_max_y = i.force_max_y
						force_min_y = i.force_min_y
						use_servo_limit_z = i.use_servo_limit_z   
						force_max_z = i.force_max_z
						force_min_z = i.force_min_z
						proportional_coefficient = i.proportional_coefficient   
						integral_coefficient = i.integral_coefficient
						derivate_coefficient = i.derivate_coefficient
												   
						file.write("		actuator.linear_velocity.x = "  + "%s" % linear_velocity_x)   
						file.write("\n")
						file.write("		actuator.linear_velocity.y = "  + "%s" % linear_velocity_y)
						file.write("\n")
						file.write("		actuator.linear_velocity.z = "  + "%s" % linear_velocity_z)
						file.write("\n")
						file.write("		actuator.use_local_linear_velocity = "  + "%s" % use_local_linear_velocity)
						file.write("\n")
						file.write("		actuator.use_servo_limit_x = "  + "%s" % use_servo_limit_x)
						file.write("\n")
						file.write("		actuator.force_max_x = "  + "%s" % force_max_x)
						file.write("\n")
						file.write("		actuator.force_min_x = "  + "%s" % force_min_x)   
						file.write("\n")
						file.write("		actuator.use_servo_limit_y = "  + "%s" % use_servo_limit_y)
						file.write("\n")
						file.write("		actuator.force_max_y = "  + "%s" % force_max_y)
						file.write("\n")
						file.write("		actuator.force_min_y = "  + "%s" % force_min_y)
						file.write("\n")
						file.write("		actuator.use_servo_limit_z = "  + "%s" % use_servo_limit_z)
						file.write("\n")
						file.write("		actuator.force_max_z = "  + "%s" % force_max_z)
						file.write("\n")
						file.write("		actuator.force_min_z = "  + "%s" % force_min_z)
						file.write("\n")
						file.write("		actuator.proportional_coefficient = "  + "%s" % proportional_coefficient)
						file.write("\n")
						file.write("		actuator.integral_coefficient = "  + "%s" % integral_coefficient)
						file.write("\n")
						file.write("		actuator.derivate_coefficient = "  + "%s" % derivate_coefficient) 
						file.write("\n")
						file.write("\n")			
						
					if mode == 'OBJECT_CHARACTER':
						offset_location_x = i.offset_location.x
						offset_location_y = i.offset_location.y   
						offset_location_z = i.offset_location.z
						use_local_location = i.use_local_location
						use_add_character_location = i.use_add_character_location	 
						offset_rotation_x = i.offset_rotation.x
						offset_rotation_y = i.offset_rotation.y   
						offset_rotation_z = i.offset_rotation.z
						use_local_rotation = i.use_local_rotation
						use_character_jump = i.use_character_jump	  
																													 
						file.write("		actuator.offset_location.x = "  + "%s" % offset_location_x)   
						file.write("\n")
						file.write("		actuator.offset_location.y = "  + "%s" % offset_location_y)
						file.write("\n")
						file.write("		actuator.offset_location.z = "  + "%s" % offset_location_z)
						file.write("\n")
						file.write("		actuator.use_local_location = "  + "%s" % use_local_location)
						file.write("\n")
						file.write("		actuator.use_add_character_location = "  + "%s" % use_add_character_location)   
						file.write("\n")
						file.write("		actuator.offset_rotation.x = "  + "%s" % offset_rotation_x)
						file.write("\n")
						file.write("		actuator.offset_rotation.y = "  + "%s" % offset_rotation_y)
						file.write("\n")
						file.write("		actuator.offset_rotation.z = "  + "%s" % offset_rotation_z)   
						file.write("\n")
						file.write("		actuator.use_local_rotation = "  + "%s" % use_local_rotation)
						file.write("\n")
						file.write("		actuator.use_character_jump = "  + "%s" % use_character_jump)		   
						file.write("\n")
						file.write("\n")
				#End Writing Motion Actuator Data
				#########################################################################   
						

				#########################################################################
				#Get Parent Actuator Data If Actuator Type is = Parent
				if type == 'PARENT':
						
					name = i.name
					pin = i.pin
					active = i.active
					mode = i.mode		  
					#End Getting Parent Actuator Data
					

					#Start Writing Parent Data to File
					file.write("		bpy.ops.logic.actuator_add(type = " + "'" + type + "'," " name = " + "'" + name + "' " + "+ " + "\"" + "%s" + "\"" + " % " + "+ " + "change_name " + ")")
					file.write("\n")
					file.write("		actuator = act_obj.game.actuators"  + "["  '-1'  "]")
					file.write("\n")
					file.write("		actuator.show_expanded = "  + "%s" % expanded)
					file.write("\n")	
					file.write("		actuator.pin = "  + "%s" % pin)
					file.write("\n")
					file.write("		actuator.active = "  + "%s" % active)
					file.write("\n")				
					file.write("		actuator.mode = " + "'"  + "%s" % mode + "'")   
					file.write("\n")


					if mode == 'SETPARENT':
						use_compound = i.use_compound
						use_ghost = i.use_ghost   

						file.write("		actuator.use_compound = "  + "%s" % use_compound)
						file.write("\n")
						file.write("		actuator.use_ghost = "  + "%s" % use_ghost)   
						file.write("\n")
						file.write("\n")						
				#End Writing Parent Actuator Data
				######################################################################### 


				#########################################################################
				#Get Property Actuator Data If Actuator Type is = Property
				if type == 'PROPERTY':
						
					name = i.name
					pin = i.pin
					active = i.active
					mode = i.mode		  
					#End Getting Property Actuator Data
					

					#Start Writing Property Data to File
					file.write("		bpy.ops.logic.actuator_add(type = " + "'" + type + "'," " name = " + "'" + name + "' " + "+ " + "\"" + "%s" + "\"" + " % " + "+ " + "change_name " + ")")
					file.write("\n")
					file.write("		actuator = act_obj.game.actuators"  + "["  '-1'  "]")
					file.write("\n")
					file.write("		actuator.show_expanded = "  + "%s" % expanded)
					file.write("\n")	
					file.write("		actuator.pin = "  + "%s" % pin)
					file.write("\n")
					file.write("		actuator.active = "  + "%s" % active)
					file.write("\n")				
					file.write("		actuator.mode = " + "'"  + "%s" % mode + "'")   
					file.write("\n")

					if mode == 'ASSIGN' or mode == 'ADD':
						property = i.property
						value = i.value   

						file.write("		actuator.property = " + "'"  + "%s" % property + "'")
						file.write("\n")
						file.write("		actuator.value = " + "'"  + "%s" % value + "'")   
						file.write("\n")
						file.write("\n")

					if mode == 'TOGGLE' or mode == 'LEVEL' or mode == 'COPY':
						property = i.property   

						file.write("		actuator.property = " + "'"  + "%s" % property + "'")
						file.write("\n")
						file.write("\n")			
				#End Writing Property Actuator Data
				#########################################################################
				
				
				#########################################################################
				#Get Random Actuator Data If Actuator Type is = Random
				if type == 'RANDOM':
						
					name = i.name
					pin = i.pin
					active = i.active
					distribution = i.distribution		  
					#End Getting Random Actuator Data
					

					#Start Writing Random Data to File
					file.write("		bpy.ops.logic.actuator_add(type = " + "'" + type + "'," " name = " + "'" + name + "' " + "+ " + "\"" + "%s" + "\"" + " % " + "+ " + "change_name " + ")")
					file.write("\n")
					file.write("		actuator = act_obj.game.actuators"  + "["  '-1'  "]")
					file.write("\n")
					file.write("		actuator.show_expanded = "  + "%s" % expanded)
					file.write("\n")	
					file.write("		actuator.pin = "  + "%s" % pin)
					file.write("\n")
					file.write("		actuator.active = "  + "%s" % active)
					file.write("\n")				
					file.write("		actuator.distribution = " + "'"  + "%s" % distribution + "'")   
					file.write("\n")	
							 
					if distribution == 'BOOL_CONSTANT':
						seed = i.seed   
						property = i.property
						use_always_true = i.use_always_true
									
						file.write("		actuator.seed = "  + "%s" % seed)
						file.write("\n")				
						file.write("		actuator.property = " + "'"  + "%s" % property + "'")   
						file.write("\n")
						file.write("		actuator.use_always_true = "  + "%s" % use_always_true)   
						file.write("\n")
						file.write("\n")			
						
					if distribution == 'BOOL_UNIFORM':  
						seed = i.seed   
						property = i.property
									
						file.write("		actuator.seed = "  + "%s" % seed)
						file.write("\n")				
						file.write("		actuator.property = " + "'"  + "%s" % property + "'")   
						file.write("\n")
						file.write("\n")
												
					if distribution == 'BOOL_BERNOUILLI':
						seed = i.seed   
						property = i.property
						chance = i.chance
									
						file.write("		actuator.seed = "  + "%s" % seed)
						file.write("\n")				
						file.write("		actuator.property = " + "'"  + "%s" % property + "'")   
						file.write("\n")
						file.write("		actuator.chance = "  + "%s" % chance)   
						file.write("\n")
						file.write("\n")
						
					if distribution == 'INT_CONSTANT':
						seed = i.seed   
						property = i.property
						value = i.int_value
									
						file.write("		actuator.seed = "  + "%s" % seed)
						file.write("\n")				
						file.write("		actuator.property = " + "'"  + "%s" % property + "'")   
						file.write("\n")
						file.write("		actuator.int_value = "  + "%s" % value)   
						file.write("\n")
						file.write("\n")
						
					if distribution == 'INT_UNIFORM':
						seed = i.seed   
						property = i.property
						int_min = i.int_min
						int_max = i.int_max
												
						file.write("		actuator.seed = "  + "%s" % seed)
						file.write("\n")				
						file.write("		actuator.property = " + "'"  + "%s" % property + "'")   
						file.write("\n")
						file.write("		actuator.int_min = "  + "%s" % int_min)
						file.write("\n")
						file.write("		actuator.int_max = "  + "%s" % int_max)   
						file.write("\n")
						file.write("\n")			
									
					if distribution == 'INT_POISSON':
						seed = i.seed   
						property = i.property
						int_mean = i.int_mean
												
						file.write("		actuator.seed = "  + "%s" % seed)
						file.write("\n")				
						file.write("		actuator.property = " + "'"  + "%s" % property + "'")   
						file.write("\n")
						file.write("		actuator.int_mean = "  + "%s" % int_mean)	 
						file.write("\n")
						file.write("\n")						 
									
					if distribution == 'FLOAT_CONSTANT':
						seed = i.seed   
						property = i.property
						float_value = i.float_value
									
						file.write("		actuator.seed = "  + "%s" % seed)
						file.write("\n")				
						file.write("		actuator.property = " + "'"  + "%s" % property + "'")   
						file.write("\n")
						file.write("		actuator.float_value = "  + "%s" % float_value)   
						file.write("\n")
						file.write("\n")
						
					if distribution == 'FLOAT_UNIFORM':
						seed = i.seed   
						property = i.property
						float_min = i.float_min
						float_max = i.float_max
												
						file.write("		actuator.seed = "  + "%s" % seed)
						file.write("\n")				
						file.write("		actuator.property = " + "'"  + "%s" % property + "'")   
						file.write("\n")
						file.write("		actuator.float_min = "  + "%s" % float_min)
						file.write("\n")
						file.write("		actuator.float_max = "  + "%s" % float_max)   
						file.write("\n")
						file.write("\n")
						
					if distribution == 'FLOAT_NORMAL':
						seed = i.seed   
						property = i.property
						float_mean = i.float_mean
						standard_derivation = i.standard_derivation
												
						file.write("		actuator.seed = "  + "%s" % seed)
						file.write("\n")				
						file.write("		actuator.property = " + "'"  + "%s" % property + "'")   
						file.write("\n")
						file.write("		actuator.float_mean = "  + "%s" % float_mean)
						file.write("\n")
						file.write("		actuator.standard_derivation = "  + "%s" % standard_derivation)   
						file.write("\n")
						file.write("\n")			
									 
					if distribution == 'FLOAT_NEGATIVE_EXPONENTIAL':
						seed = i.seed   
						property = i.property
						half_life_time = i.half_life_time
												
						file.write("		actuator.seed = "  + "%s" % seed)
						file.write("\n")				
						file.write("		actuator.property = " + "'"  + "%s" % property + "'")   
						file.write("\n")
						file.write("		actuator.half_life_time = "  + "%s" % half_life_time)	 
						file.write("\n")
						file.write("\n")			
				#End Writing Random Actuator Data
				#########################################################################
				
				
				#########################################################################
				#Get Scene Actuator Data If Actuator Type is = Scene
				if type == 'SCENE':
						
					name = i.name
					pin = i.pin
					active = i.active
					mode = i.mode		  
					#End Getting Scene Actuator Data
					

					#Start Writing Scene Data to File
					file.write("		bpy.ops.logic.actuator_add(type = " + "'" + type + "'," " name = " + "'" + name + "' " + "+ " + "\"" + "%s" + "\"" + " % " + "+ " + "change_name " + ")")
					file.write("\n")
					file.write("		actuator = act_obj.game.actuators"  + "["  '-1'  "]")
					file.write("\n")
					file.write("		actuator.show_expanded = "  + "%s" % expanded)
					file.write("\n")	
					file.write("		actuator.pin = "  + "%s" % pin)
					file.write("\n")
					file.write("		actuator.active = "  + "%s" % active)
					file.write("\n")				
					file.write("		actuator.mode = " + "'"  + "%s" % mode + "'")   
					file.write("\n")
					file.write("\n")			
				#End Writing Scene Actuator Data
				#########################################################################
				
				
				#########################################################################
				#Get Steering Actuator Data If Actuator Type is = Steering
				if type == 'STEERING':
						
					name = i.name
					pin = i.pin
					active = i.active
					mode = i.mode
					distance = i.distance
					velocity = i.velocity
					acceleration = i.acceleration
					turn_speed = i.turn_speed   
					facing = i.facing
					facing_axis = i.facing_axis  
					normal_up = i.normal_up  
					self_terminated = i.self_terminated
					lock_z_velocity = i.lock_z_velocity  
					show_visualization = i.show_visualization												   
					#End Getting Steering Actuator Data
					

					#Start Writing Steering Data to File
					file.write("		bpy.ops.logic.actuator_add(type = " + "'" + type + "'," " name = " + "'" + name + "' " + "+ " + "\"" + "%s" + "\"" + " % " + "+ " + "change_name " + ")")
					file.write("\n")
					file.write("		actuator = act_obj.game.actuators"  + "["  '-1'  "]")
					file.write("\n")
					file.write("		actuator.show_expanded = "  + "%s" % expanded)
					file.write("\n")	
					file.write("		actuator.pin = "  + "%s" % pin)
					file.write("\n")
					file.write("		actuator.active = "  + "%s" % active)
					file.write("\n")				
					file.write("		actuator.mode = " + "'"  + "%s" % mode + "'")
					file.write("\n")	
					file.write("		actuator.distance = "  + "%s" % distance)
					file.write("\n")
					file.write("		actuator.velocity = "  + "%s" % velocity)
					file.write("\n")	
					file.write("		actuator.acceleration = "  + "%s" % acceleration)
					file.write("\n")
					file.write("		actuator.turn_speed = "  + "%s" % turn_speed)
					file.write("\n")
					file.write("		actuator.facing = "  + "%s" % facing)
					file.write("\n")
					file.write("		actuator.facing_axis = " + "'"  + "%s" % facing_axis + "'")
					file.write("\n")
					file.write("		actuator.normal_up = "  + "%s" % normal_up)
					file.write("\n")
					file.write("		actuator.self_terminated = "  + "%s" % self_terminated)
					file.write("\n")
					file.write("		actuator.lock_z_velocity = "  + "%s" % lock_z_velocity)
					file.write("\n")
					file.write("		actuator.show_visualization = "  + "%s" % show_visualization)   
					file.write("\n")
					file.write("\n")
							
					if mode == 'PATHFOLLOWING':
						update_period = i.update_period  
												
						file.write("		actuator.update_period = "  + "%s" % update_period)  
						file.write("\n")
						file.write("\n")					
				#End Writing Steering Actuator Data
				#########################################################################
				
						 
				#########################################################################   
				#Get Sound Actuator Data If Actuator Type is = Sound
				if type == 'SOUND':
						
					name = i.name
					pin = i.pin
					active = i.active												   
					#End Getting Sound Actuator Data
					

					#Start Writing Sound Data to File
					file.write("		bpy.ops.logic.actuator_add(type = " + "'" + type + "'," " name = " + "'" + name + "' " + "+ " + "\"" + "%s" + "\"" + " % " + "+ " + "change_name " + ")")
					file.write("\n")
					file.write("		actuator = act_obj.game.actuators"  + "["  '-1'  "]")
					file.write("\n")
					file.write("		actuator.show_expanded = "  + "%s" % expanded)
					file.write("\n")	
					file.write("		actuator.pin = "  + "%s" % pin)
					file.write("\n")
					file.write("		actuator.active = "  + "%s" % active)
					file.write("\n")
					file.write("\n")
				#End Writing Sound Actuator Data
				#########################################################################
				
				
				#########################################################################   
				#Get State Actuator Data If Actuator Type is = State
				if type == 'STATE':
						
					name = i.name
					pin = i.pin
					active = i.active
					operation = i.operation
					state0 = i.states[0]
					state1 = i.states[1]
					state2 = i.states[2]
					state3 = i.states[3]
					state4 = i.states[4]
					state5 = i.states[5]
					state6 = i.states[6]
					state7 = i.states[7]
					state8 = i.states[8]
					state9 = i.states[9]
					state10 = i.states[10]
					state11 = i.states[11]
					state12 = i.states[12]
					state13 = i.states[13]
					state14 = i.states[14]
					state15 = i.states[15]
					state16 = i.states[16]
					state17 = i.states[17]
					state18 = i.states[18]
					state19 = i.states[19]
					state20 = i.states[20]
					state21 = i.states[21]
					state22 = i.states[22]
					state23 = i.states[23]
					state24 = i.states[24]
					state25 = i.states[25]
					state26 = i.states[26]
					state27 = i.states[27]
					state28 = i.states[28]
					state29 = i.states[29]									
					#End Getting State Actuator Data
					

					#Start Writing State Data to File
					file.write("		bpy.ops.logic.actuator_add(type = " + "'" + type + "'," " name = " + "'" + name + "' " + "+ " + "\"" + "%s" + "\"" + " % " + "+ " + "change_name " + ")")
					file.write("\n")
					file.write("		actuator = act_obj.game.actuators"  + "["  '-1'  "]")
					file.write("\n")
					file.write("		actuator.show_expanded = "  + "%s" % expanded)
					file.write("\n")	
					file.write("		actuator.pin = "  + "%s" % pin)
					file.write("\n")
					file.write("		actuator.active = "  + "%s" % active)
					file.write("\n")
					file.write("		actuator.operation = " + "'"  + "%s" % operation + "'")
					file.write("\n")
					file.write("		actuator.states[0] = "  + "%s" % state0)
					file.write("\n")
					file.write("		actuator.states[1] = "  + "%s" % state1)
					file.write("\n")
					file.write("		actuator.states[2] = "  + "%s" % state2)
					file.write("\n")
					file.write("		actuator.states[3] = "  + "%s" % state3)
					file.write("\n")
					file.write("		actuator.states[4] = "  + "%s" % state4)
					file.write("\n")
					file.write("		actuator.states[5] = "  + "%s" % state5)
					file.write("\n")
					file.write("		actuator.states[6] = "  + "%s" % state6)
					file.write("\n")
					file.write("		actuator.states[7] = "  + "%s" % state7)
					file.write("\n")
					file.write("		actuator.states[8] = "  + "%s" % state8)
					file.write("\n")
					file.write("		actuator.states[9] = "  + "%s" % state9)
					file.write("\n")
					file.write("		actuator.states[10] = "  + "%s" % state10)
					file.write("\n")
					file.write("		actuator.states[11] = "  + "%s" % state11)
					file.write("\n")
					file.write("		actuator.states[12] = "  + "%s" % state12)
					file.write("\n")
					file.write("		actuator.states[13] = "  + "%s" % state13)
					file.write("\n")
					file.write("		actuator.states[14] = "  + "%s" % state14)
					file.write("\n")
					file.write("		actuator.states[15] = "  + "%s" % state15)
					file.write("\n")
					file.write("		actuator.states[16] = "  + "%s" % state16)
					file.write("\n")
					file.write("		actuator.states[17] = "  + "%s" % state17)
					file.write("\n")
					file.write("		actuator.states[18] = "  + "%s" % state18)
					file.write("\n")
					file.write("		actuator.states[19] = "  + "%s" % state19)
					file.write("\n")
					file.write("		actuator.states[20] = "  + "%s" % state20)
					file.write("\n")
					file.write("		actuator.states[21] = "  + "%s" % state21)
					file.write("\n")
					file.write("		actuator.states[22] = "  + "%s" % state22)
					file.write("\n")
					file.write("		actuator.states[23] = "  + "%s" % state23)
					file.write("\n")
					file.write("		actuator.states[24] = "  + "%s" % state24)
					file.write("\n")
					file.write("		actuator.states[25] = "  + "%s" % state25)
					file.write("\n")
					file.write("		actuator.states[26] = "  + "%s" % state26)
					file.write("\n")
					file.write("		actuator.states[27] = "  + "%s" % state27)
					file.write("\n")
					file.write("		actuator.states[28] = "  + "%s" % state28)
					file.write("\n")
					file.write("		actuator.states[29] = "  + "%s" % state29)  
					file.write("\n")							   
					file.write("\n")
				#End Writing State Actuator Data
				#########################################################################   
					
				
				#########################################################################   
				#Get Visibility Actuator Data If Actuator Type is = Visibility
				if type == 'VISIBILITY':
						
					name = i.name
					pin = i.pin
					active = i.active
					use_visible = i.use_visible
					use_occlusion = i.use_occlusion  
					apply_to_children = i.apply_to_children 
					#End Getting Visibility Actuator Data
					

					#Start Writing Visibility Data to File
					file.write("		bpy.ops.logic.actuator_add(type = " + "'" + type + "'," " name = " + "'" + name + "' " + "+ " + "\"" + "%s" + "\"" + " % " + "+ " + "change_name " + ")")
					file.write("\n")
					file.write("		actuator = act_obj.game.actuators"  + "["  '-1'  "]")
					file.write("\n")
					file.write("		actuator.show_expanded = "  + "%s" % expanded)
					file.write("\n")	
					file.write("		actuator.pin = "  + "%s" % pin)
					file.write("\n")
					file.write("		actuator.active = "  + "%s" % active)
					file.write("\n")
					file.write("		actuator.use_visible = "  + "%s" % use_visible)
					file.write("\n")	
					file.write("		actuator.use_occlusion = "  + "%s" % use_occlusion)
					file.write("\n")
					file.write("		actuator.apply_to_children = "  + "%s" % apply_to_children)  
					file.write("\n")
					file.write("\n")
		#End Writing Visibility Actuator Data
		#########################################################################
		file.write("\n")	
		file.write("		#Link sensors to controllers")
		file.write("\n")
		#Link Sensors to Controllers 
		#########################################################################
		act_obj = bpy.context.active_object

		for i in act_obj.game.controllers:
			con_name = i.name
			controller_name = ("+" + con_name + "+")
			con = act_obj.game.controllers[con_name]
				
			for i in act_obj.game.sensors:
				sens_name = i.name
				sens = act_obj.game.sensors[sens_name]
				
				if sens_name.find(controller_name) != -1:								  
					
					file.write("		#Link " + "%s" % sens_name + " " + "sensor" + " " + "to" + " " + "%s" % con_name  + " " + "controller") 
					file.write("\n")
					file.write("		sens = act_obj.game.sensors" + "[" + "'" + sens_name + "' " + "+ " + "\"" + "%s" + "\"" + " % " + "+ " + "change_name " + "]")
					file.write("\n")
					file.write("		con = act_obj.game.controllers" + "[" + "'" + con_name + "' " + "+ " + "\"" + "%s" + "\"" + " % " + "+ " + "change_name " + "]")  
					file.write("\n")
					file.write("		sens.link(con)")		
					file.write("\n")		
					file.write("\n")
		#End of Linking Sensors to Controllers
		#########################################################################
		file.write("\n")	
		file.write("		#Link actuators to controllers")
		file.write("\n")
		#Link Actuators to Controllers 
		#########################################################################
		act_obj = bpy.context.active_object

		for i in act_obj.game.controllers:
			con_name = i.name
			controller_name = ("+" + con_name + "+")
			con = act_obj.game.controllers[con_name]						
				
			for i in act_obj.game.actuators:
				act_name = i.name
				act = act_obj.game.actuators[act_name]									 
					
				if act_name.find(controller_name) != -1:				

					file.write("		#Link " + "%s" % act_name + " " + "actuator" + " " + "to" + " " + "%s" % con_name  + " " + "controller")				
					file.write("\n")
					file.write("		act = act_obj.game.actuators" + "[" + "'" + act_name + "' " + "+ " + "\"" + "%s" + "\"" + " % " + "+ " + "change_name " + "]")
					file.write("\n")
					file.write("		con = act_obj.game.controllers" + "[" + "'" + con_name + "' " + "+ " + "\"" + "%s" + "\"" + " % " + "+ " + "change_name " + "]")  
					file.write("\n")
					file.write("		act.link(con)")  
					file.write("\n")		
					file.write("\n")
		#End of Linking Actuators to Controllers
		#########################################################################


		#########################################################################
		file.write("		return {'FINISHED'}")   
		file.write("\n")
		file.write("\n")
		file.write("\n")
		file.write("\n")			
		file.write("def menu_item(self, context):\n")
		file.write("	   self.layout.operator(" + logic_name + ".bl_idname, text=\"" + logic_name + "\",icon=\"LOGIC\")\n")
		file.write("\n")
		file.write("def register():\n")
		file.write("	   bpy.utils.register_module(__name__)\n")
		file.write("	   bpy.types.LOGIC_copy_to_state.append(menu_item)\n")
		file.write("\n")
		file.write("def unregister():\n")
		file.write("	   bpy.utils.unregister_module(__name__)\n")
		file.write("	   bpy.types.LOGIC_copy_to_state.remove(menu_item)\n")
		file.write("\n")
		file.write("if __name__ == \"__main__\":\n")
		file.write("	   register()\n")


								
		return {'FINISHED'}
######################################################################################
  
 
		
def register():
	bpy.utils.register_class(LogicCopySetup)
	bpy.utils.register_class(SaveCopy)   
	bpy.utils.register_class(Copy_Links_State)
	bpy.utils.register_class(LogicCopyOperator)
	bpy.utils.register_class(ErrorCopyOperator)
	bpy.utils.register_class(TipCopyOperator)   
	bpy.types.Object.logic_brick_copy = bpy.props.StringProperty(
		name = "Save As",
		description = "Name for saved game logic",
		default = ''
	  )
	bpy.types.Object.save_sign = bpy.props.StringProperty(
		name = "Save Symbol",
		description = "Only save logic bricks with this symbol(Default is + even if blank)",
		default = '+'
	  )  
	bpy.types.Object.add_to_state = bpy.props.IntProperty(
		name = "State",
		description = "The state to add logic bricks to",
		min = 1 ,
		max = 30
	  )
	bpy.types.Object.change_name = bpy.props.IntProperty(
		name = "Logic Names",
		description = "Change number by 1, if you need to add the setup again",
		min = 1 ,
	  )  
	
def unregister():
	bpy.utils.unregister_class(LogicCopySetup)
	bpy.utils.unregister_class(SaveCopy)   
	bpy.utils.unregister_class(Copy_Links_State)
	bpy.utils.unregister_class(LogicCopyOperator)
	bpy.utils.unregister_class(ErrorCopyOperator)
	bpy.utils.unregister_class(TipCopyOperator)
	del bpy.types.Object.logic_brick_copy   
	del bpy.types.Object.add_to_state
	del bpy.types.Object.save_sign
	del bpy.types.Object.change_name  
	
if __name__ == "__main__":
	register()

